﻿/**
* ******************************************************
*                                                       *
*   Copyright (C) Microsoft. All rights reserved.       *
*                                                       *
*******************************************************
*/
/// <reference path="../typings/node.d.ts" />
/// <reference path="../typings/Q.d.ts" />
/// <reference path="../typings/mkdirp.d.ts"/>
/// <reference path="../typings/ncp.d.ts"/>
var fs = require("fs");
var Q = require("q");
var mkdirp = require("mkdirp");
var ncp = require("ncp");
var path = require("path");
var crypto = require('crypto');

var VCRemoteUtility;
(function (VCRemoteUtility) {
    var UtilHelper = (function () {
        function UtilHelper() {
        }
        /**
        * Converts an untyped argument into a boolean in a "sensible" way, treating only the string "true" as true rather than any non-empty string
        *
        * @param {any} input Any object that we want to determine its truthiness
        *
        * @return {boolean} For strings, this is whether the string is case-insensitively equal to "true", otherwise it is javascript's interpretation of truthiness
        */
        UtilHelper.argToBool = function (input) {
            if (typeof input === "string") {
                return input.toLowerCase() === "true";
            }

            return !!input;
        };

        UtilHelper.readFileContentsSync = function (filename, encoding) {
            var contents = fs.readFileSync(filename, (encoding || "utf-8"));
            if (contents) {
                contents = contents.replace(/^\uFEFF/, ""); // Windows is the BOM
            }

            return contents;
        };

        UtilHelper.copyFile = function (from, to, encoding) {
            var deferred = Q.defer();
            var newFile = fs.createWriteStream(to, { encoding: encoding });
            var oldFile = fs.createReadStream(from, { encoding: encoding });
            newFile.on("finish", function () {
                deferred.resolve({});
            });
            newFile.on("error", function (e) {
                deferred.reject(e);
            });
            oldFile.on("error", function (e) {
                deferred.reject(e);
            });
            oldFile.pipe(newFile);
            return deferred.promise;
            /*
            // The original code here stripped out byte order markers (but also potentially mangle binary files)
            var contents = readFileContentsSync(from, encoding);
            fs.writeFileSync(to, contents, { encoding: encoding });
            */
        };

        // Recursively copy 'source' to 'target'
        UtilHelper.copyRecursive = function (source, target) {
            var deferred = Q.defer();
            var options = {};

            ncp.ncp(source, target, options, function (error) {
                if (error) {
                    deferred.reject(error);
                } else {
                    deferred.resolve({});
                }
            });

            return deferred.promise;
        };

        /**
        * Extract optional arguments from an arguments array.
        *
        * @param {IArguments} functionArguments The "arguments" from another function
        * @param {number} startFrom The offset where the optional arguments begin
        *
        * @returns {any[]} If functionArguments[startFrom] is an array, then that is returned. Otherwise the functionArguments array except for the first startFrom elements.
        */
        UtilHelper.getOptionalArgsArrayFromFunctionCall = function (functionArguments, startFrom) {
            if (functionArguments.length <= startFrom) {
                return null;
            }

            if (Array.isArray(functionArguments[startFrom])) {
                return functionArguments[startFrom];
            }

            return Array.prototype.slice.apply(functionArguments, [startFrom]);
        };

        /**
        * Synchronously create a directory if it does not exist
        *
        * @param {string} dir The directory to create
        *
        * @returns {boolean} If the directory needed to be created then returns true, otherwise returns false. If the directory could not be created, then throws an exception.
        */
        UtilHelper.createDirectoryIfNecessary = function (dir) {
            if (!fs.existsSync(dir)) {
                try  {
                    mkdirp.sync(dir);
                    return true;
                } catch (err) {
                    // if multiple msbuild processes are running on a first time solution build, another one might have created the basedir. check again.
                    if (!fs.existsSync(dir)) {
                        throw err;
                    }
                }
            }

            return false;
        };

        UtilHelper.findUniqueDirectoryName = function (dir) {
            var id = "";
            var directoryName;
            do {
                id = crypto.randomBytes(12).toString("hex").slice(0, 12);
                directoryName = path.join(dir, id);
            } while(fs.existsSync(directoryName));
            return directoryName;
        };

        UtilHelper.isValidCordovaAppName = function (str) {
            for (var i = 0, n = str.length; i < n; i++) {
                var code = str.charCodeAt(i);
                if (code < 32 || UtilHelper.InvalidAppNameChars[code]) {
                    return false;
                }
            }

            return true;
        };

        UtilHelper.invalidAppNameCharacters = function () {
            return Object.keys(UtilHelper.InvalidAppNameChars).map(function (c) {
                return UtilHelper.InvalidAppNameChars[c];
            });
        };

        UtilHelper.quotesAroundIfNecessary = function (filename) {
            return (filename.indexOf(" ") > -1) ? "\"" + filename + "\"" : filename;
        };
        UtilHelper.InvalidAppNameChars = {
            34: "\"",
            36: "$",
            38: "&",
            39: "/",
            60: "<",
            92: "\\"
        };
        return UtilHelper;
    })();
    VCRemoteUtility.UtilHelper = UtilHelper;
})(VCRemoteUtility || (VCRemoteUtility = {}));

module.exports = VCRemoteUtility;
